#
#   Humerus - Game Shell
#

import os
from pygame import Rect
from albow.shell import Shell
from albow.controls import Label, Button, Row, Column, Image
from albow.dialogs import alert, ask
from albow.file_dialogs import request_old_filename, request_new_filename
from albow.resource import get_font, get_image
from screen import Screen, menu_button
from choose_level import ChooseLevelScreen
from play import LevelScreen
from editor import EditorMenuScreen, EditorLevelScreen
from game import Game
from level import Level
import settings
from settings import options

class HumerusShell(Shell):

	def __init__(self, display):
		Shell.__init__(self, display)
		self.redraw_every_frame = False
		self.game = Game()
		self.menu_screen = MenuScreen(self)
		self.story_screen = StoryScreen(self)
		self.choose_level_screen = ChooseLevelScreen(self)
		self.level_screen = LevelScreen(self)
		self.editor_menu_screen = EditorMenuScreen(self)
		self.editor_level_screen = EditorLevelScreen(self)
		self.set_timer(settings.frame_time)
		level_file_passed = self.handle_filename_option(options.filename)
		if options.edit:
			if not self.game.current_level:
				self.edit_new_level()
			else:
				self.show_editor_level()
		else:
			if level_file_passed:
				self.start_level()
			else:
				self.show_menu()
	
	def handle_filename_option(self, path):
		if path:
			path = os.path.abspath(path)
			if os.path.isdir(path):
				self.game.current_level_dir = path
			elif path.endswith(Level.level_file_suffix):
				self.game.current_level_dir = os.path.dirname(path)
				self.load_level_file(path)
				return True
			else:
				alert("%s is not a directory or recognised file type."
					% path)
	
	def show_menu(self):
		self.show_screen(self.menu_screen)
	
	def show_story(self):
		self.show_screen(self.story_screen)
	
	def show_level(self):
		self.show_screen(self.level_screen)
	
	def start_tutorial(self):
		self.start_game(tutorial = True)
	
	def new_game(self):
		self.start_game(tutorial = False)
	
	def start_game(self, tutorial):
		paths = self.game.get_current_playing_level_list(tutorial)
		if not paths:
			alert("There are no levels in %s." % 
				self.game.get_current_playing_level_dir())
			return
		self.load_file(paths[0])
		self.start_level()
	
	def start_level(self, testing = False):
		self.game.start_level(testing)
		self.show_level()
	
	def prev_level(self, confirm = False):
		self.change_level(-1, confirm, "Return to previous level?")
	
	def next_level(self):
		self.change_level(1)
	
	def change_level(self, way, confirm = False, prompt = None):
		path = self.game.get_adjacent_level_path(way)
		if path:
			if confirm:
				if ask(prompt, ["OK", "Cancel"]) <> "OK":
					return
			self.start_level_file(path)
		else:
			self.show_menu()
	
	def start_level_file(self, path):
		if self.load_level_file(path):
			self.start_level()
	
	def enter_editor(self):
		self.game.abort_level()
		self.show_screen(self.editor_menu_screen)
	
	def show_editor_menu(self):
		self.show_screen(self.editor_menu_screen)
	
	def show_editor_level(self):
		level = self.game.current_level
		if level:
			level.reset()
		self.show_screen(self.editor_level_screen)
	
	def edit_new_level(self):
		if not self.ask_save_level():
			return
		self.game.create_level()
		self.show_editor_level()
	
	def quit(self):
		if self.ask_save_level():
			Shell.quit(self)
	
	def load_file(self, path):
		if path.endswith(Game.game_file_suffix):
			self.load_game_file(path)
		elif path.endswith(Level.level_file_suffix):
			self.load_level_file(path)
		else:
			alert("%s is not a recognised file type." % path)
		
	def load_game(self):
		if not self.ask_save_game():
			return
		if not self.ask_save_level():
			return
		directory = self.game.get_current_save_dir()
		path = request_old_filename(suffixes = [Game.game_file_suffix],
			directory = directory)
		if path:
			self.game.current_saved_game_dir = os.path.dirname(path)
			if self.load_game_file(path):
				next_level = self.game.next_uncompleted_level()
				if next_level:
					self.load_level_file(next_level)
					self.start_level()
				else:
					alert("You have completed all the levels in this saved game.")
	
	def load_game_file(self, path):
		try:
			return self.game.load_game(path)
		except EnvironmentError, e:
			alert("Unable to load game. %s" % e)
	
	def save_game(self):
		game = self.game
		old_path = game.saved_game_pathname
		old_dir = None
		old_name = None
		if old_path:
			old_dir, old_name = os.path.split(old_path)
		if not old_dir:
			old_dir = self.game.get_current_save_dir()
		if not old_name:
			old_name = "Game"
		path = request_new_filename(prompt = "Save Game As:",
			suffix = Game.game_file_suffix,
			directory = old_dir, filename = old_name)
		if path:
			game.saved_game_pathname = path
			try:
				game.save_game(path)
			except EnvironmentError, e:
				alert("Unable to save game. %s" % e)
			return True
		else:
			return False
	
	def load_level(self):
		if not self.ask_save_level():
			return
		directory = self.game.get_current_custom_level_dir()
		path = request_old_filename(suffixes = [Level.level_file_suffix],
			directory = directory)
		if path:
			old_level_dir = self.game.current_level_dir
			new_level_dir = os.path.dirname(path)
			if new_level_dir <> old_level_dir:
				if not self.ask_save_game(
					"Save progress before changing level directories?"):
						return
			self.load_level_file(path)
	
	def load_level_file(self, path):
		if not self.ask_save_level():
			return False
		try:
			self.game.load_level(path)
		except EnvironmentError, e:
			alert("Unable to load level. %s" % e)
			return False
		return True
	
	def save_level_as(self):
		if self.request_level_filename():
			self.write_level()
	
	def save_level(self):
		if not self.game.current_level.pathname:
			if not self.request_level_filename():
				return False
		self.write_level()
		return True
	
	def write_level(self):
		try:
			self.game.save_level()
		except EnvironmentError, e:
			alert("Unable to save level. %s" % e)
	
	def request_level_filename(self):
		level = self.game.current_level
		old_path = level.pathname
		old_dir = None
		old_name = None
		if old_path:
			old_dir, old_name = os.path.split(old_path)
		std_level_dir = settings.get_default_playing_level_dir()
		if old_dir == std_level_dir and not options.edit_std_levels:
			old_dir = settings.get_default_editor_level_dir()
		if not old_dir:
			old_dir = self.game.get_current_custom_level_dir()
		path = request_new_filename(prompt = "Save Level As:",
			suffix = Level.level_file_suffix,
			directory = old_dir, filename = old_name)
		if path:
			level.pathname = path
			return True
		else:
			return False

	def ask_save_game(self, prompt = "Save progress in current game?"):
		if self.game.unsaved_progress:
			response = ask(prompt,
				["Save", "Discard", "Cancel"])
			if response == "Cancel":
				return False
			if response == "Save":
				if not self.save_game():
					return False
		return True
	
	def ask_save_level(self):
		level = self.game.current_level
		if level and level.changed:
			name = level.get_name()
			response = ask("Save changes to '%s'?" % name,
				["Save", "Discard", "Cancel"])
			if response == "Cancel":
				return False
			if response == "Save":
				if not self.save_level():
					return False
		return True
	
	def choose_level(self):
		self.show_screen(self.choose_level_screen)
	
	def play_std_levels(self):
		level_dir = settings.get_default_playing_level_dir()
		self.game.set_current_level_dir(level_dir)
		self.start_game(tutorial = False)
	
	def get_current_custom_level_set_dir(self):
		old_dir = self.game.get_current_custom_level_dir()
		if old_dir.endswith(Game.level_set_suffix):
			old_dir = os.path.dirname(old_dir)
		return old_dir
	
	def play_custom_levels(self):
		old_dir = self.get_current_custom_level_set_dir()
		path = request_old_filename(suffixes = [Game.level_set_suffix,
			Level.level_file_suffix], directory = old_dir)
		if path:
			if path.endswith(Level.level_file_suffix):
				self.load_level_file(path)
				self.start_level()
			else:
				self.game.set_current_level_dir(path)
				self.start_game(tutorial = False)
	
	def new_level_set(self):
		old_dir = self.get_current_custom_level_set_dir()
		path = request_new_filename("Create Level Set As:",
			directory = old_dir, suffix = Game.level_set_suffix)
		if path:
			try:
				self.game.create_level_set(path)
			except EnvironmentError, e:
				alert("%s: Can't create level set: %s" % (path, e))
			else:
				self.game.set_current_level_dir(path)


class MenuScreen(Screen):

	def __init__(self, shell):
		Screen.__init__(self, shell)
		game = shell.game
		def mb(color, *args):
			b = menu_button(*args)
			b.fg_color = color
			return b
		y = (255, 255, 0)
		g = (128, 255, 0)
		b = (0, 128, 255)
		title = self.create_title()
		items1 = [
			mb(y, "About",           shell.show_story),
			mb(y, "Tutorial",        shell.start_tutorial),
			mb(y, "New Game",        shell.new_game),
			mb(y, "Load Game",       shell.load_game),
			mb(y, "Save Game",       shell.save_game,         game.save_game_enabled),
			mb(y, "Quit",            shell.quit),
		]
		gap = Label("")
		items2 = [
			mb(g, "Start Level",     shell.start_level,       game.start_enabled),
			mb(g, "Resume Level",    shell.show_level,        game.resume_enabled),
			mb(g, "Abort Level",     game.abort_level,        game.abort_enabled),
			gap,
			mb(b, "Choose Level...",       shell.choose_level),
			mb(b, "Play Standard Levels",  shell.play_std_levels,    game.std_levels_enabled),
			mb(b, "Play Custom Levels...", shell.play_custom_levels),
			mb(b, "Level Editor", shell.enter_editor),
		]
		col1 = Column(items1, align = 'l')
		col2 = Column(items2, align = 'l', height = col1.rect.height,
			expand = gap)
		menu = Row([col1, col2], spacing = 100, align = 't')
		contents = Column([title, menu], spacing = 20)
		contents.rect.center = self.rect.center
		self.add(contents)
	
	def create_title(self):
		raise NotImplementedError


class StoryScreen(Screen):

	def __init__(self, shell):
		Screen.__init__(self, shell)
		items = [
			self.create_story(),
			menu_button("Back", action = shell.show_menu),
		]
		box = Column(items, align = 'c', spacing = 30)
		box.rect.center = self.rect.center
		self.add(box)

	def create_story(self):
		raise NotImplementedError
